/*************************************************************/
/*                                                           */
/* newton.c                                                  */ 
/*                                                           */
/* Newton fractal program.                                   */
/*                                                           */
/* By Stefan Spaennare, Lund, Sweden                         */
/*                                                           */
/* E-mail: stefan@spaennare.se                               */
/*                                                           */
/* First version: July 1992                                  */
/* Latest update: 2006-02-06                                 */
/*                                                           */
/*************************************************************/

/****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "image.h"


double timetic=(double)(CLOCKS_PER_SEC);


void cadd(ar,ai,br,bi,cr,ci)
double *ar,*ai;
double br,bi,cr,ci;
{
   *ar=br+cr;
   *ai=bi+ci;
} /* cadd */


void cmul(ar,ai,br,bi,cr,ci)
double *ar,*ai;
double br,bi,cr,ci;
{
   *ar=br*cr-bi*ci;
   *ai=br*ci+bi*cr;
} /* cmul */


void cdiv(ar,ai,br,bi,cr,ci)
double *ar,*ai;
double br,bi,cr,ci;
{
   double temp,dr,di;

   temp=cr*cr+ci*ci;
   cmul(&dr,&di,br,bi,cr,-ci);
   *ar=dr/temp;
   *ai=di/temp;
} /* cdiv */


int iter(cr,ci,itmax)
double cr,ci;
int itmax;
{
   double zr,zi,tr,ti,sr,si,eps;
   int it;

   zr=cr;
   zi=ci;
   eps=1.0e-12;
   it=0;

   while ((it < itmax) && (fabs(zr-1.0) > eps) && (fabs(zi) > eps)) {
      cmul(&tr,&ti,zr,zi,zr,zi);
      cmul(&sr,&si,tr,ti,zr,zi);
      sr=2.0*sr+1.0;
      si=2.0*si;
      tr=3.0*tr;
      ti=3.0*ti;
      cdiv(&zr,&zi,sr,si,tr,ti);
      it++;
   } 
   return(it);
} /* end iter */


int main(argc,argv)
int argc;
char *argv[];
{

   FILE *infile;

   struct image_head header;    
   short **image;

   double a,b,da,db,ca,cb,wa,wb,flops,ftime,eps;
   int i,j,itmax,it,xsize,ysize;
   clock_t time1,timediff;


   if (argc != 3) {
      printf("Usage: %s image newtonfile\n",argv[0]);
      printf("\n");
      printf("Parameters: a wa b wb itmax xsize ysize\n"); 
      exit(0);
   } /* if */


   if ((infile = fopen(argv[2],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */


   fscanf(infile,"%lf %lf %lf %lf %d %d %d",&a,&wa,&b,&wb,&itmax,&xsize,&ysize); 

   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=ysize;
   header.wid=xsize;
 
   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);



da=wa/(double)(xsize);
db=wb/(double)(ysize);

printf("\n");
printf("Newton fractal\n");
printf("\n");
printf("itmax= %6d; xsize= %6d; ysize= %6d\n",itmax,xsize,ysize);
printf("a= %18.16f; wa= %18.16f\n",a,wa);
printf("b= %18.16f; wb= %18.16f\n",b,wb);
printf("\n");


eps=1.0e-12;
flops=0.0;

time1=clock();

for (i=0; i<ysize; i++) {
   if ((i % 100) == 0)
      printf("%4d\n",i); 
   for (j=0; j<xsize; j++) {
      ca=a+da*(double)(j);
      cb=b+db*(double)(ysize-i-1);
      if (fabs(ca-1.0) < eps) {
         ca=a+da*(double)(j+1);
      } /* if */
      if (fabs(cb) < eps) {
         cb=b+db*(double)(ysize-i);
      } /* if */
      it=iter(ca,cb,itmax); 
      image[i][j]=it;
      flops=flops+28.0*(double)(it)+7.0;
   } /* for j */
} /* for i */


timediff=clock()-time1;

ftime=(double)(timediff)/timetic;
flops=flops/ftime;

printf("\n");
printf("CPU-time: %9.2f s\n",ftime);
printf("Mflops:   %9.2f\n",flops/1.0e6);
printf("\n");

WriteImage(argv[1],image,header,im_s16);

fclose(infile);

} /* End */
